@@ -1,6 +1,5 @@
-mhinge <- function(xtr, ytr, xte=NULL, yte=NULL, cost = NULL, nu=0.1, learner="ls", m1=200, K=10, cv1=FALSE, twin = FALSE, m2=200, cv2=FALSE){
+mhinge <- function(xtr, ytr, xte=NULL, yte=NULL, cost = NULL, nu=0.1, learner=c("tree", "ls", "sm"), maxdepth=1, m1=200, twinboost=FALSE, m2=200){
   call <- match.call()
-                                        #  if(m1 < 1 || m2 < 1) stop("Too small m1 or m2\n")
   n1 <- length(ytr)
   ncla <- length(unique(ytr))
   if(min(ytr)!=1 || max(ytr)!=ncla)
@@ -24,24 +23,17 @@ mhinge <- function(xtr, ytr, xte=NULL, yte=NULL, cost = NULL, nu=0.1, learner="l
     if(!is.null(xte) && !is.null(yte))
       z.te[which(yte==k), k] <- 1
     cost1 <- cost.mul[k]
-    m1up <- m1
-    if(cv1){
-      eval(parse(text = paste("tr",k, " <- cv.bst(x = xtr, y = z[,k], K=K, cost=cost.mul[k], learner=learner, ctrl = bst_control(mstop=m1, nu=nu))", sep=""))) 
-      m1up <- eval(parse(text = paste("which.min(tr",k, "$cv)", sep="")))
-    } 
-    eval(parse(text = paste("tr",k, " <- bst(x = xtr, y = z[,k],  cost=cost.mul[k], learner=learner, ctrl = bst_control(mstop=m1up, nu=nu))", sep=""))) 
-    if(twin){
-      m2up <- m2
-      if(cv2){
-        eval(parse(text = paste("cvtr <- cv.bst(x = xtr, y = z[,k], K=K, cost=cost.mul[k], learner=learner, ctrl = bst_control(twinboost=TRUE, f.init=predict(tr", k, "), xselect.init = tr", k, "$xselect, mstop=m2, nu=nu))", sep=""))) 
-        m2up <- eval(parse(text = paste("which.min(cvtr$cv)", sep="")))
-      }
-      eval(parse(text = paste("tr",k, " <- bst(x = xtr, y = z[,k],  cost=cost.mul[k], learner=learner, ctrl = bst_control(twinboost=TRUE, f.init=predict(tr", k, "), xselect.init = tr", k, "$xselect, mstop=m2up, nu=nu))", sep=""))) 
+    eval(parse(text = paste("tr",k, " <- bst(x = xtr, y = z[,k], cost=cost.mul[k], learner=learner, ctrl = bst_control(mstop=m1, nu=nu), control.tree=list(maxdepth=maxdepth))", sep=""))) 
+    if(twinboost){
+      eval(parse(text = paste("tr",k, " <- bst(x = xtr, y = z[,k],  cost=cost.mul[k], learner=learner, ctrl = bst_control(twinboost=TRUE, f.init=predict(tr", k, "), xselect.init = tr", k, "$xselect, mstop=m2, nu=nu), control.tree=list(maxdepth=maxdepth))", sep=""))) 
     } 
     fit.tr[[k]] <- eval(parse(text=paste("predict(tr",k,", type='all.res')", sep="")))           
+    if(learner!="tree" || maxdepth==1) 
     fpar[[k]] <- eval(parse(text=paste("fpartial.bst(tr",k,")", sep="")))           
     xsel <- c(xsel, eval(parse(text=paste("tr",k,"$xselect", sep=""))))           
     ensemble[[k]] <- eval(parse(text=paste("tr",k,"$ensemble", sep="")))           
+    if(k==1)
+      mstop1 <- eval(parse(text=paste("tr",k,"$ctrl$mstop", sep="")))           
     if(!is.null(xte)){
       fit.te[[k]] <- eval(parse(text=paste("predict(tr",k,", newdata=xte, type='all.res')", sep=""))) 
       if(!is.null(yte)){
@@ -49,15 +41,42 @@ mhinge <- function(xtr, ytr, xte=NULL, yte=NULL, cost = NULL, nu=0.1, learner="l
       }
     }
   }
-  mstop1 <- tr1$ctrl$mstop 
   err.tr <- unlist(lapply(1:mstop1, function(j){tmp <- NULL; for(i in 1:ncla) tmp <- cbind(tmp, fit.tr[[i]][,j]); 1/length(ytr) * sum(ytr != apply(tmp, 1, which.max)) }))
-  if(!is.null(yte) && !is.null(xte))
+  if(!is.null(yte) && !is.null(xte)){
     err.te <- unlist(lapply(1:mstop1, function(j){tmp <- NULL; for(i in 1:ncla) tmp <- cbind(tmp, fit.te[[i]][,j]); 1/length(yte) * sum(yte != apply(tmp, 1, which.max)) }))
-  RET=list(call=call, learner=learner, nu=nu, twin=twin, m1=m1, m2=m2, cv1=cv1, cv2=cv2, risk.te=risk.te, err.tr = err.tr, err.te = err.te, ensemble = ensemble, xsel=round(table(xsel)/ncla,2), fpar=fpar)
+}
+  RET=list(call=call, learner=learner, nu=nu, twinboost=twinboost, m1=m1, m2=m2, risk.te=risk.te, err.tr = err.tr, err.te = err.te, ensemble = ensemble, xsel=round(table(xsel)/ncla,2), fpar=fpar)
   class(RET) = "mhinge"
   return(RET)
 }
 
+cv.mhinge <- function(x, y, balance=FALSE, K=10, cost = NULL, nu=0.1, learner=c("tree", "ls", "sm"), maxdepth=1, m1=200, twinboost = FALSE, m2=200, trace=FALSE, plot.it = TRUE, se = TRUE, ...)
+{
+  call <- match.call()
+  learner <- match.arg(learner)
+  if(balance)
+  all.folds <- balanced.folds(y, K)
+  else all.folds <- cv.folds(length(y), K)
+  fraction <- 1:m1
+#  fraction <- seq(from = 1, to = m1, by=5)
+  residmat <- matrix(0, length(fraction), K)
+  for(i in seq(K)) {
+    if(trace)
+      cat("\n CV Fold", i, "\n\n")
+    omit <- all.folds[[i]]
+    fit <- mhinge(xtr = x[ - omit,,drop=FALSE  ], ytr = y[ - omit], xte = x[ omit,,drop=FALSE ], yte=y[ omit ], cost = cost, nu=nu, learner = learner, maxdepth=maxdepth, m1=m1, twinboost=twinboost, m2=m2)
+###cross validation misclassification error
+    residmat[,i] <- fit$err.te
+  if(trace && i==K)
+  cat("End of cross-validation\n")
+  }
+  cv <- apply(residmat, 1, mean)
+  cv.error <- sqrt(apply(residmat, 1, var)/K)
+  object<-list(residmat = residmat, fraction = fraction, cv = cv, cv.error = cv.error)
+  if(plot.it) plotCVbst(object,se=se)
+  invisible(object)
+}
+
 print.mhinge <- function(x, ...) {
 
   cat("\n")
@@ -69,7 +88,7 @@ print.mhinge <- function(x, ...) {
   cat("Step size: ", x$nu, "\n")
   if(!x$cv1)
     cat("Number of boosting iterations: mstop =", x$m1, "\n")
-  if(x$twin){
+  if(x$twinboost){
     cat("Twin boosting", "\n")
     if(!x$cv2)
       cat("Number of twin boosting iterations: mstop =", x$m2, "\n")
