@@ -1,159 +1,205 @@
-    msvm_fit <- function(x, y, Fboost, yv, lq, b, learner, twinboost=FALSE, f.init=NULL, xselect.init=NULL, maxdepth=1, nu=0.1, df=4, inde=inde){
-    p <- dim(x)[2]
-    k <- length(table(y))
-    ind <- coef <- rep(NA, k)
-    if(learner=="tree")
-     if(maxdepth == 1) xselect <- rep(NA, k)
-     else xselect <- vector("list", k)
-    mse.w <- coef.w <- matrix(NA,ncol=p, nrow=k)
-    cor.w <- matrix(0, ncol=p, nrow=k)
-    nob <- c(1:k)[-b]
-    u <- -lq[,-b]*(sign(Fboost[,-b] - yv[,-b]) + 1)/2
-    u1 <- lq[,b]*(sign(-apply(as.matrix(Fboost[,-b]), 1, sum) - yv[,b]) + 1)/2 ### sum-to-zero constraint
-    
-    u <- u + u1
-    u <- as.matrix(u)
-    tmp <- matrix(NA, nrow=dim(x)[1], ncol=k)
-    tmp[,-b] <- u
-    u <- tmp
-    if(!twinboost) xselect.init <- 1:p
-    ml.fit <- vector(mode = "list", length = k)
-    pred.tr <- matrix(NA, nrow=dim(x)[1], ncol=k)
-    coef0 <- matrix(NA, ncol(x), nrow=k)
-    if(learner=="ls"){
-      for (j in xselect.init){
-        coef0[,j] <- 1/sum(x[,j]^2)*apply(as.matrix(x[,j] * u), 2, sum)
-        for(i in nob)
-          pred.tr[,i] <- x[,j] * coef0[i,j] 
+msvm_fit <- function(x, y, Fboost, yv, lq, b, learner, twinboost=FALSE, f.init=NULL, xselect.init=NULL, fixed.depth=TRUE, n.term.node=6, maxdepth=1, nu=0.1, df=4, inde=inde){
+  p <- dim(x)[2]
+  k <- length(table(y))
+  ind <- coef <- rep(NA, k)
+  if(learner=="tree")
+    if(maxdepth == 1) xselect <- rep(NA, k)
+    else xselect <- vector("list", k)
+  mse.w <- coef.w <- matrix(NA,ncol=p, nrow=k)
+  cor.w <- matrix(0, ncol=p, nrow=k)
+  nob <- c(1:k)[-b]
+  u <- -lq[,-b]*(sign(Fboost[,-b] - yv[,-b]) + 1)/2
+  u1 <- lq[,b]*(sign(-apply(as.matrix(Fboost[,-b]), 1, sum) - yv[,b]) + 1)/2 ### sum-to-zero constraint
+  
+  u <- u + u1
+  u <- as.matrix(u)
+  tmp <- matrix(NA, nrow=dim(x)[1], ncol=k)
+  tmp[,-b] <- u
+  u <- tmp
+  if(!twinboost) xselect.init <- 1:p
+  ml.fit <- vector(mode = "list", length = k)
+  pred.tr <- matrix(NA, nrow=dim(x)[1], ncol=k)
+  coef0 <- matrix(NA, ncol(x), nrow=k)
+  if(learner=="ls"){
+    for (j in xselect.init){
+      coef0[,j] <- 1/sum(x[,j]^2)*apply(as.matrix(x[,j] * u), 2, sum)
+      for(i in nob)
+        pred.tr[,i] <- x[,j] * coef0[i,j] 
+      ss <- apply(pred.tr^2, 2, sum)
+      if(twinboost){   
+        a <- 2*colSums(u*pred.tr) - ss
+        for(i in nob){
+          if(!all(pred.tr[,i] == 0))
+            cor.w[i,j] <- cov(f.init[,i],pred.tr[,i])/sqrt(sum(pred.tr[,i]^2))
+          mse.w[i,j] <- cor.w[i,j]^2 * a[i]
+        }
+      }
+      else mse.w[,j] <- 2*colSums(u*pred.tr) - ss
+    }
+    for(i in nob)
+      ind[i] <- which.max(mse.w[i,])
+    for(i in nob){
+      ml.fit[[i]] <- lm(u[,i] ~ x[, ind[i]] - 1)
+      coef[i] <- coef(ml.fit[[i]])  ### this should be the same as above
+    }
+    xselect <- ind
+  }
+  else
+    if(learner=="sm"){
+      for(j in xselect.init){
+###Twin L2 Boosting with genral weak learner, Buhlmann, page 8, step 4, in Twin boosting, improved feature selection andprediction
+        for(i in nob){
+          if(length(unique(x[,j])) < 4)
+            res.fit <- lm(u[,i] ~ x[,j] - 1)
+          else res.fit <- smooth.spline(x=x[,j],y=u[,i],df=df)
+          pred.tr[,i] <- fitted(res.fit)
+        }
         ss <- apply(pred.tr^2, 2, sum)
         if(twinboost){   
           a <- 2*colSums(u*pred.tr) - ss
           for(i in nob){
             if(!all(pred.tr[,i] == 0))
-            cor.w[i,j] <- cov(f.init[,i],pred.tr[,i])/sqrt(sum(pred.tr[,i]^2))
+              cor.w[i,j] <- cov(f.init[,i],pred.tr[,i])/sqrt(sum(pred.tr[,i]^2))
             mse.w[i,j] <- cor.w[i,j]^2 * a[i]
           }
         }
         else mse.w[,j] <- 2*colSums(u*pred.tr) - ss
-      }
-        for(i in nob)
+      }   
+      for(i in nob)
         ind[i] <- which.max(mse.w[i,])
+### this can be optimized with the previous results res.fit
       for(i in nob){
-        ml.fit[[i]] <- lm(u[,i] ~ x[, ind[i]] - 1)
-        coef[i] <- coef(ml.fit[[i]])  ### this should be the same as above
+        if(length(unique(x[,ind[i]])) < 4)
+          ml.fit[[i]] <- lm(u[,i] ~ x[,ind[i]] - 1)
+        else ml.fit[[i]] <- smooth.spline(x=x[,ind[i]],y=u[,i],df=df)
       }
       xselect <- ind
     }
     else
-      if(learner=="sm"){
-        for(j in xselect.init){
-###Twin L2 Boosting with genral weak learner, Buhlmann, page 8, step 4, in Twin boosting, improved feature selection andprediction
+      if(learner=="tree"){
+        cntrl <- rpart.control(maxdepth = maxdepth, #minsplit = nsample-1, #minbucket = 1,
+                               maxsurrogate = 0, maxcompete = 0, #usesurrogate=0
+                               cp = 0, xval = 0)
+        cntrl0 <- rpart.control(maxdepth = 6, #minsplit = nsample-1, #minbucket = 1,
+                                maxsurrogate = 0, maxcompete = 0, #usesurrogate=0
+                                cp = 0, xval = 2)
+        if(!twinboost){
           for(i in nob){
-            if(length(unique(x[,j])) < 4)
-              res.fit <- lm(u[,i] ~ x[,j] - 1)
-            else res.fit <- smooth.spline(x=x[,j],y=u[,i],df=df)
-            pred.tr[,i] <- fitted(res.fit)
+            data.tr <- as.data.frame(cbind(u[,i],x)); 
+            colnames(data.tr) <- c("u",colnames(x))
+            if(fixed.depth)
+              ml.fit[[i]] <- rpart(u~.,data=data.tr,method="anova",control=cntrl)
+            else{
+              treefit <- rpart(u~.,data=data.tr,method="anova",control=cntrl0)
+### find nsplit <= n.term.node
+### (note: nsplit + 1 = number of terminal node = tree size) 
+              tmp <- which(treefit$cptable[,"nsplit"] <= n.term.node - 1)
+### find the largest one among them
+              tmp1 <- tmp[length(tmp)]
+### prune the tree to desired number of splits, which has the desirgd n.term.node 
+              ml.fit[[i]] <- prune(treefit, cp=treefit$cptable[,"CP"][which(treefit$cptable[,"nsplit"]==tmp1)])
+            }
+            labs <- rownames(ml.fit[[i]][["splits"]])
+            if(!is.null(labs))
+              xselect[[i]] <- which(colnames(x) %in% labs)
           }
-          ss <- apply(pred.tr^2, 2, sum)
-          if(twinboost){   
+        }
+        else{
+          for(j in 1:nrow(inde)){
+            for(i in nob){
+              if(maxdepth==1){
+                data.tr <- as.data.frame(cbind(u[,i],x[,j])); 
+                colnames(data.tr) <- c("u",colnames(x)[j])
+              }
+              else{
+                warnings("Twin HingeBoost with base learner trees has not been fully tested\n")
+                data.tr <- as.data.frame(cbind(u[,i],x[,inde[j,]])); 
+                colnames(data.tr) <- c("u",colnames(x)[inde[j,]])
+              }
+###Twin L2 Boosting with genral weak learner, Buhlmann, page 127, step 4, in Twin boosting, improved feature selection and prediction, Statistics and Computing (2007) Volume: 20, Issue: 2, Pages: 119-138
+              if(fixed.depth)
+                res.fit <- rpart(u~.,data=data.tr,method="anova",control=cntrl)
+              else{
+                treefit <- rpart(u~.,data=data.tr,method="anova",control=cntrl0)
+### find nsplit <= n.term.node
+### (note: nsplit + 1 = number of terminal node = tree size) 
+                tmp <- which(treefit$cptable[,"nsplit"] <= n.term.node - 1) 
+### find the largest one among them
+                tmp1 <- tmp[length(tmp)]
+### prune the tree to desired number of splits, which has the desirgd n.term.node 
+                res.fit <- prune(treefit, cp=treefit$cptable[,"CP"][which(treefit$cptable[,"nsplit"]==tmp1)])
+              }
+              pred.tr[,i] <- predict(res.fit)
+            }  
+            ss <- apply(pred.tr^2, 2, sum)
             a <- 2*colSums(u*pred.tr) - ss
             for(i in nob){
               if(!all(pred.tr[,i] == 0))
-              cor.w[i,j] <- cov(f.init[,i],pred.tr[,i])/sqrt(sum(pred.tr[,i]^2))
+                cor.w[i,j] <- cov(f.init[,i],pred.tr[,i])/sqrt(sum(pred.tr[,i]^2))
               mse.w[i,j] <- cor.w[i,j]^2 * a[i]
             }
           }
-          else mse.w[,j] <- 2*colSums(u*pred.tr) - ss
-        }   
-        for(i in nob)
-        ind[i] <- which.max(mse.w[i,])
+          for(i in nob)
+            ind[i] <- which.max(mse.w[i,])
 ### this can be optimized with the previous results res.fit
-        for(i in nob){
-          if(length(unique(x[,ind[i]])) < 4)
-            ml.fit[[i]] <- lm(u[,i] ~ x[,ind[i]] - 1)
-          else ml.fit[[i]] <- smooth.spline(x=x[,ind[i]],y=u[,i],df=df)
-        }
-        xselect <- ind
-      }
-      else
-        if(learner=="tree"){
-          cntrl <- rpart.control(maxdepth = maxdepth, #minsplit = nsample-1, #minbucket = 1,
-                                 maxsurrogate = 0, maxcompete = 0, #usesurrogate=0
-                                 cp = 0, xval = 0)
-          if(!twinboost){
+          if(maxdepth==1){
             for(i in nob){
-              data.tr <- as.data.frame(cbind(u[,i],x)); 
-              colnames(data.tr) <- c("u",colnames(x))
-              ml.fit[[i]] <- rpart(u~.,data=data.tr,method="anova",control=cntrl)
-              labs <- rownames(ml.fit[[i]][["splits"]])
-              if(!is.null(labs))
-                xselect[[i]] <- which(colnames(x) %in% labs)
+              data.tr <- as.data.frame(cbind(u[,i],x[,ind[i]])); 
+              colnames(data.tr) <- c("u",colnames(x)[ind[i]])
+              treefit <- rpart(u~.,data=data.tr,method="anova",control=cntrl0)
+              if(fixed.depth)
+                ml.fit[[i]] <- rpart(u~.,data=data.tr,method="anova",control=cntrl)
+              else{
+### find nsplit <= n.term.node
+### (note: nsplit + 1 = number of terminal node = tree size) 
+                tmp <- which(treefit$cptable[,"nsplit"] <= n.term.node - 1) 
+### find the largest one among them
+                tmp1 <- tmp[length(tmp)]
+### prune the tree to desired number of splits, which has the desirgd n.term.node 
+                ml.fit[[i]] <- prune(treefit, cp=treefit$cptable[,"CP"][which(treefit$cptable[,"nsplit"]==tmp1)])
+              }
             }
+            xselect[[i]] <- ind
           }
-          else{
-            for(j in 1:nrow(inde)){
-              for(i in nob){
-                if(maxdepth==1){
-                  data.tr <- as.data.frame(cbind(u[,i],x[,j])); 
-                  colnames(data.tr) <- c("u",colnames(x)[j])
-                }
-                else{
-                  warnings("Twin HingeBoost with base learner trees has not been fully tested\n")
-                  data.tr <- as.data.frame(cbind(u[,i],x[,inde[j,]])); 
-                  colnames(data.tr) <- c("u",colnames(x)[inde[j,]])
-                }
-###Twin L2 Boosting with genral weak learner, Buhlmann, page 8, step 4, in Twin boosting, improved feature selection and prediction
-### cf weakboostadapt.rpart in twin.R by Buhlmann, received from Horton in 2009
-                res.fit <- rpart(u~.,data=data.tr,method="anova",control=cntrl)
-                pred.tr[,i] <- predict(res.fit)
-              }  
-              ss <- apply(pred.tr^2, 2, sum)
-              a <- 2*colSums(u*pred.tr) - ss
-              for(i in nob){
-                if(!all(pred.tr[,i] == 0))
-                cor.w[i,j] <- cov(f.init[,i],pred.tr[,i])/sqrt(sum(pred.tr[,i]^2))
-                mse.w[i,j] <- cor.w[i,j]^2 * a[i]
-              }
-            }
-        for(i in nob)
-        ind[i] <- which.max(mse.w[i,])
-### this can be optimized with the previous results res.fit
-            if(maxdepth==1){
-              for(i in nob){
-                data.tr <- as.data.frame(cbind(u[,i],x[,ind[i]])); 
-                colnames(data.tr) <- c("u",colnames(x)[ind[i]])
+          else {
+            tmp1 <- NULL
+            for(i in nob){
+              data.tr <- as.data.frame(cbind(u[,i],x[,ind[i,]])); 
+              colnames(data.tr) <- c("u",colnames(x)[ind[i,]])
+              if(fixed.depth)
                 ml.fit[[i]] <- rpart(u~.,data=data.tr,method="anova",control=cntrl)
-              }
-              xselect[[i]] <- ind
+              else{
+                treefit <- rpart(u~.,data=data.tr,method="anova",control=cntrl0)
+### find nsplit <= n.term.node
+### (note: nsplit + 1 = number of terminal node = tree size) 
+                tmp <- which(treefit$cptable[,"nsplit"] <= n.term.node - 1) 
+### find the largest one among them
+                tmp1 <- tmp[length(tmp)]
+### prune the tree to desired number of splits, which has the desirgd n.term.node 
+                ml.fit[[i]] <- prune(treefit, cp=treefit$cptable[,"CP"][which(treefit$cptable[,"nsplit"]==tmp1)])
+              } 
+              tmp <- ml.fit[[i]]$frame$var[ml.fit[[i]]$frame$var%in%colnames(x)]
+              tmp1 <- c(tmp1, tmp)
             }
-            else {
-              tmp1 <- NULL
-              for(i in nob){
-                data.tr <- as.data.frame(cbind(u[,i],x[,ind[i,]])); 
-                colnames(data.tr) <- c("u",colnames(x)[ind[i,]])
-                ml.fit[[i]] <- rpart(u~.,data=data.tr,method="anova",control=cntrl)
-                tmp <- ml.fit[[i]]$frame$var[ml.fit[[i]]$frame$var%in%colnames(x)]
-                tmp1 <- c(tmp1, tmp)
-              }
-              tmp1 <- unique(tmp1)
-              if(length(tmp1)!=0)
-                xselect[[i]] <- as.character(tmp1)
-            }
-          }          
-        }
+            tmp1 <- unique(tmp1)
+            if(length(tmp1)!=0)
+              xselect[[i]] <- as.character(tmp1)
+          }
+        }          
+      }
 ### update prediction
-    for(i in nob){
-       if(learner=="sm")
-        Fboost[,i] <- Fboost[,i] + nu * fitted(ml.fit[[i]])
-       else
-        Fboost[,i] <- Fboost[,i] + nu * predict(ml.fit[[i]])
-    }
-    Fboost[,b] <- -apply(as.matrix(Fboost[,-b]), 1, sum) ### sum-to-zero
+  for(i in nob){
+    if(learner=="sm")
+      Fboost[,i] <- Fboost[,i] + nu * fitted(ml.fit[[i]])
+    else
+      Fboost[,i] <- Fboost[,i] + nu * predict(ml.fit[[i]])
+  }
+  Fboost[,b] <- -apply(as.matrix(Fboost[,-b]), 1, sum) ### sum-to-zero
 ### empirical loss
-    risk <- loss.msvm(y, Fboost, k)
-    ensemble <- xselect
-    return(list(b=b, Fboost=Fboost, ens=ml.fit, risk=risk, xselect=xselect, coef=coef))
-    } 
+  risk <- loss.msvm(y, Fboost, k)
+  ensemble <- xselect
+  return(list(b=b, Fboost=Fboost, ens=ml.fit, risk=risk, xselect=xselect, coef=coef))
+} 
 
 loss.msvm <- function(y, f, k, type=c("total","all"), cost=NULL){
   type <- match.arg(type)
@@ -167,12 +213,12 @@ loss.msvm <- function(y, f, k, type=c("total","all"), cost=NULL){
   los <- matrix(los, byrow=FALSE, ncol=k)
   tmp <- lq * los 
   if(type=="total")
-  return(sum(tmp)/length(y))
+    return(sum(tmp)/length(y))
   else return(tmp/length(y))
 }
 
 #######################################################################################################################################################
-msvm <- function(x,y, cost=NULL, family = c("hinge"), ctrl = bst_control(), control.tree=list(maxdepth=1), learner=c("ls", "sm", "tree")){
+msvm <- function(x,y, cost=NULL, family = c("hinge"), ctrl = bst_control(), control.tree=list(fixed.depth=TRUE, n.term.node=6, maxdepth=1), learner=c("ls", "sm", "tree")){
   call <- match.call()
   if(any(y < 1)) stop("y must > 0 \n")
   family <- match.arg(family)
@@ -218,7 +264,10 @@ msvm <- function(x,y, cost=NULL, family = c("hinge"), ctrl = bst_control(), cont
   coef <- matrix(NA, ncol=k, nrow=mstop)
   xselect <- vector("list", mstop)
   ind <- rep(NA, k)
+  fixed.depth <- control.tree$fixed.depth
+  if(is.null(fixed.depth)) fixed.depth <- TRUE
   maxdepth <- control.tree$maxdepth
+  n.term.node <- control.tree$n.term.node
 
   if(learner=="tree" && twinboost){
     if(maxdepth==1){
@@ -241,13 +290,13 @@ msvm <- function(x,y, cost=NULL, family = c("hinge"), ctrl = bst_control(), cont
   while (m <= mstop){
     tmp <- rep(NA, k); res <- vector("list", k)
     for(i in 1:k){
-    res[[i]] <- msvm_fit(x=x, y=y, Fboost=Fboost, yv=yv, lq=lq, b=i, learner=learner, twinboost=twinboost, f.init=f.init, xselect.init=xselect.init, maxdepth=maxdepth, nu=nu, df=df, inde=inde)
-    tmp[i] <- res[[i]]$risk
+      res[[i]] <- msvm_fit(x=x, y=y, Fboost=Fboost, yv=yv, lq=lq, b=i, learner=learner, twinboost=twinboost, f.init=f.init, xselect.init=xselect.init, fixed.depth=fixed.depth, n.term.node=n.term.node, maxdepth=maxdepth, nu=nu, df=df, inde=inde)
+      tmp[i] <- res[[i]]$risk
     }
     optb <- which.min(tmp)
     Fboost <- res[[optb]]$Fboost
     for(i in 1:k)
-    ens[[m,i]] <- res[[optb]]$ens[i]
+      ens[[m,i]] <- res[[optb]]$ens[i]
     risk[m] <- res[[optb]]$risk
     xselect[[m]] <- (res[[optb]]$xselect)
     coef[m,] <- res[[optb]]$coef
@@ -269,14 +318,14 @@ predict.msvm <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=c("re
     mstop <- object$ctrl$mstop
   else if(mstop > object$ctrl$mstop)
     stop("mstop must be equal or smaller than the one used for estimation ", object$ctrl$mstop)
-#  if((type=="loss" || type=="error") && (is.null(newdata) || is.null(newy)))
-#    stop("For estimation of loss or error, both newdata and newy are needed\n")
+                                        #  if((type=="loss" || type=="error") && (is.null(newdata) || is.null(newy)))
+                                        #    stop("For estimation of loss or error, both newdata and newy are needed\n")
   type <- match.arg(type)
   one <- rep(1,nrow(object$x))
   x <- object$x
   y <- object$y
   if(is.null(newdata) && is.null(newy))
-  ynow <- y
+    ynow <- y
   else ynow <- newy
   if(!missing(newdata)){
     if(object$ctrl$center){
@@ -292,8 +341,8 @@ predict.msvm <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=c("re
   if(missing(newdata)) p <- dim(x)[1]
   else{ 
     if(!missing(newy))
-     if(dim(newdata)[1] != length(newy))
-    stop("Number of rows of newdata is different from length of newy\n")
+      if(dim(newdata)[1] != length(newy))
+        stop("Number of rows of newdata is different from length of newy\n")
     newdata <- as.matrix(newdata)
     p <- dim(newdata)[1]
   }
@@ -313,16 +362,16 @@ predict.msvm <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=c("re
         if(learner=="tree")  
           lp[,i] <- lp[,i] + nu*predict(ens[[m,i]][[1]], newdata = newdata)
         else if(learner=="sm"){
-         if(length(unique(x[,object$ensemble[[m]][i]])) < 4)
-{      lp[,i] <- lp[,i] + nu * coef(ens[[m, i]][[1]])* newdata[, object$ensemble[[m]][i]]
-}         else  
-         lp[,i] <- lp[,i] + nu * predict(ens[[m, i]][[1]], newdata[, object$ensemble[[m]][i]])$y
-}        else if(learner=="ls")
+          if(length(unique(x[,object$ensemble[[m]][i]])) < 4)
+            {      lp[,i] <- lp[,i] + nu * coef(ens[[m, i]][[1]])* newdata[, object$ensemble[[m]][i]]
+                 }         else  
+          lp[,i] <- lp[,i] + nu * predict(ens[[m, i]][[1]], newdata[, object$ensemble[[m]][i]])$y
+        }        else if(learner=="ls")
           lp[,i] <- lp[,i] + nu * object$coef[m, i] * newdata[, object$ensemble[[m]][i]]
     }
     lp[,baseclass[m]] <- - rowSums(as.matrix(lp[,-baseclass[m]])) 
     if(type=="loss"){
-        risk[m] <- loss.msvm(ynow, lp, k)
+      risk[m] <- loss.msvm(ynow, lp, k)
     }
     else if(type == "error"){
       tmp <- apply(lp, 1, which.max)
@@ -342,7 +391,7 @@ predict.msvm <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=c("re
     split(sample(1:n), rep(1:folds, length = n))
   }
 "cv.msvm" <-
-  function(x, y, K = 10, cost = NULL, family = "hinge", learner = c("tree","ls", "sm"), ctrl = bst_control(), type = c("risk", "misc"), plot.it = TRUE, se = TRUE, ...)
+  function(x, y, balance=FALSE, K = 10, cost = NULL, family = "hinge", learner = c("tree","ls", "sm"), ctrl = bst_control(), type = c("risk", "misc"), plot.it = TRUE, se = TRUE, ...)
   {
     call <- match.call()
     family <- match.arg(family)
@@ -356,7 +405,9 @@ predict.msvm <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=c("re
     twinboost <- ctrl$twinboost
     trace <- ctrl$trace
     ctrl.cv <- ctrl
-    all.folds <- cv.folds(length(y), K)
+    if(balance)  
+      all.folds <- balanced.folds(y, K)
+    else all.folds <- cv.folds(length(y), K)
     fraction <- seq(mstop)
     residmat <- matrix(NA, mstop, K)
     for(i in seq(K)) {
@@ -367,8 +418,9 @@ predict.msvm <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=c("re
         ctrl.cv$f.init <- ctrl$f.init[ - omit, ]
       fit <- msvm(x[ - omit,,drop=FALSE  ], y[ - omit], cost = cost, family = family, learner = learner, ctrl = ctrl.cv, ...)
       if(type=="risk")
-      fit <- predict.msvm(fit, newdata = x[omit,  ,drop=FALSE], newy=y[ omit], mstop = mstop, type="loss")
-      else stop("Not implemented\n")
+        fit <- predict.msvm(fit, newdata = x[omit,  ,drop=FALSE], newy=y[ omit], mstop = mstop, type="loss")
+      else if(type=="misc")
+        fit <- predict.msvm(fit, newdata = x[omit,  ,drop=FALSE], newy=y[ omit], mstop = mstop, type="error")
       residmat[, i] <- fit
     }
     cv <- apply(residmat, 1, mean)
@@ -379,28 +431,30 @@ predict.msvm <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=c("re
   }
 
 "plotCVbst" <-
-  function(cv.bst.object,se=TRUE){
+  function(cv.bst.object,se=TRUE,ylab=NULL, main=NULL, width=0.02, col="darkgrey", ...){
     fraction <- cv.bst.object$fraction
     cv <- cv.bst.object$cv
     cv.error <- cv.bst.object$cv.error
-    plot(fraction, cv, type = "b", xlab = "Iteration", ylab="Cross-validation", ylim = range(cv, cv + cv.error,
-                                                                  cv - cv.error))
+    if(is.null(ylab))
+      ylab <- "Cross-validation Misclassification Error"
+    plot(fraction, cv, type = "b", xlab = "Iteration", ylab= ylab, ylim = range(cv, cv + cv.error, cv - cv.error), main=main)
     if(se)
       error.bars(fraction, cv + cv.error, cv - cv.error,
-                 width = 1/length(fraction))
-#    detach(cv.bst.object)
+                 width = width, col=col)
+                                        #                 width = 1/length(fraction), col=col)
+                                        #    detach(cv.bst.object)
 
     invisible()
   }
 
 "error.bars" <-
-  function(x, upper, lower, width = 0.02, ...)
+  function(x, upper, lower, width=0.02, col="darkgrey", ...)
   {
     xlim <- range(x)
     barw <- diff(xlim) * width
-    segments(x, upper, x, lower, ...)
-    segments(x - barw, upper, x + barw, upper, ...)
-    segments(x - barw, lower, x + barw, lower, ...)
+    segments(x, upper, x, lower, col=col, ...)
+    segments(x - barw, upper, x + barw, upper, col=col, ...)
+    segments(x - barw, lower, x + barw, lower, col=col, ...)
     range(upper, lower)
   }
 
@@ -469,7 +523,7 @@ fpartial.msvm <- function (object, mstop=NULL, newdata=NULL)
       lp[[i]][,xselect] <- lp[[i]][,xselect] + nu*predict(object$ens[[m,i]][[1]], newdata = newdata)
     }
     else if(object$learner=="sm"){
-     for(i in nob)
+      for(i in nob)
         xselect <- object$ensemble[[m]][i]
       lp[[i]][,xselect] <- lp[[i]][,xselect] + nu * predict(object$ens[[m, i]][[1]], newdata[, object$ensemble[[m]][i]])$y
     }
